/******************************************************************************
**
** FILE NAME    : dwc_otg_cil_ifx.h
** PROJECT      : Twinpass/Danube/Amazon-SE
** MODULES      : DWC OTG USB
**
** DATE         : 12 April 2007
** AUTHOR       : Sung Winder
** DESCRIPTION  : Default param value.
** COPYRIGHT    :       Copyright (c) 2007
**                      Infineon Technologies AG
**                      2F, No.2, Li-Hsin Rd., Hsinchu Science Park,
**                      Hsin-chu City, 300 Taiwan.
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Date          $Author         $Comment
** 12 April 2007   Sung Winder     Initiate Version
*******************************************************************************/
#if !defined(__DWC_OTG_CIL_IFX_H__)
#define __DWC_OTG_CIL_IFX_H__

/* TODO: What is OPT mode??? */
#define dwc_param_opt_default dwc_param_opt

/**
 * Specifies the OTG capabilities. The driver will automatically
 * detect the value for this parameter if none is specified.
 * 0 - HNP and SRP capable (default)
 * 1 - SRP Only capable
 * 2 - No HNP/SRP capable
 */
#define DWC_OTG_CAP_PARAM_HNP_SRP_CAPABLE 0
#define DWC_OTG_CAP_PARAM_SRP_ONLY_CAPABLE 1
#define DWC_OTG_CAP_PARAM_NO_HNP_SRP_CAPABLE 2
#define dwc_param_otg_cap_default dwc_param_otg_cap

/**
 * Specifies whether to use slave or DMA mode for accessing the data
 * FIFOs. The driver will automatically detect the value for this
 * parameter if none is specified.
 * 0 - Slave
 * 1 - DMA (default, if available)
 */
#define dwc_param_dma_enable_default dwc_param_dma_enable

/** The DMA Burst size (applicable only for External DMA
 * Mode). 1, 4, 8 16, 32, 64, 128, 256 (default 32)
 */
#define dwc_param_dma_burst_size_default dwc_param_dma_burst_size

/**
 * Specifies the maximum speed of operation in host and device mode.
 * The actual speed depends on the speed of the attached device and
 * the value of phy_type. The actual speed depends on the speed of the
 * attached device.
 * 0 - High Speed (default)
 * 1 - Full Speed
 */
#define DWC_SPEED_PARAM_HIGH 0
#define DWC_SPEED_PARAM_FULL 1
#define dwc_param_speed_default dwc_param_speed

/** Specifies whether low power mode is supported when attached 
 *  to a Full Speed or Low Speed device in host mode.
 * 0 - Don't support low power mode (default)
 * 1 - Support low power mode
 */
#define dwc_param_host_support_fs_ls_low_power_default dwc_param_host_support_fs_ls_low_power

/** Specifies the PHY clock rate in low power mode when connected to a
 * Low Speed device in host mode. This parameter is applicable only if
 * HOST_SUPPORT_FS_LS_LOW_POWER is enabled.  If PHY_TYPE is set to FS
 * then defaults to 6 MHZ otherwise 48 MHZ.
 *
 * 0 - 48 MHz
 * 1 - 6 MHz
 */
#define DWC_HOST_LS_LOW_POWER_PHY_CLK_PARAM_48MHZ 0
#define DWC_HOST_LS_LOW_POWER_PHY_CLK_PARAM_6MHZ 1
#define dwc_param_host_ls_low_power_phy_clk_default dwc_param_host_ls_low_power_phy_clk

/**
 * 0 - Use cC FIFO size parameters
 * 1 - Allow dynamic FIFO sizing (default)
 */
// #define dwc_param_enable_dynamic_fifo_default 1
#define dwc_param_enable_dynamic_fifo_default dwc_param_enable_dynamic_fifo

/** Total number of 4-byte words in the data FIFO memory. This 
 * memory includes the Rx FIFO, non-periodic Tx FIFO, and periodic 
 * Tx FIFOs.
 * 32 to 32768 (default 8192)
 * Note: The total FIFO memory depth in the FPGA configuration is 8192.
 */
/* TODO:Need to check out the exact value. */
#define dwc_param_data_fifo_size_default dwc_param_data_fifo_size

/** Number of 4-byte words in the Rx FIFO in device mode when dynamic 
 * FIFO sizing is enabled.
 * 16 to 32768 (default 1064)
 */
/* TODO:Need to check out the exact value. */
#define dwc_param_dev_rx_fifo_size_default dwc_param_dev_rx_fifo_size
//#if defined(CONFIG_DANUBE)
//#define dwc_param_dev_rx_fifo_size_default 1024
//#elif defined(CONFIG_AMAZON_SE)
//#define dwc_param_dev_rx_fifo_size_default 256
//#else
//#error "Please choose one platform!!"
//#endif

/** Number of 4-byte words in the non-periodic Tx FIFO in device mode 
 * when dynamic FIFO sizing is enabled.
 * 16 to 32768 (default 1024)
 */
/* TODO:Need to check out the exact value. */
#define dwc_param_dev_nperio_tx_fifo_size_default dwc_param_dev_nperio_tx_fifo_size
//#if defined(CONFIG_DANUBE)
//#define dwc_param_dev_nperio_tx_fifo_size_default 1024
//#elif defined(CONFIG_AMAZON_SE)
//#define dwc_param_dev_nperio_tx_fifo_size_default 256
//#else
//#error "Please choose one platform!!"
//#endif

/** Number of 4-byte words in each of the periodic Tx FIFOs in device
 * mode when dynamic FIFO sizing is enabled.
 * 4 to 768 (default 256)
 */
/* TODO:Need to check out the exact value. */
#define dwc_param_dev_perio_tx_fifo_size_default dwc_param_dev_perio_tx_fifo_size
//#if defined(CONFIG_DANUBE)
//#define dwc_param_dev_perio_tx_fifo_size_default 768
//#elif defined(CONFIG_AMAZON_SE)
//#define dwc_param_dev_perio_tx_fifo_size_default 256
//#else
//#error "Please choose one platform!!"
//#endif

/** Number of 4-byte words in the Rx FIFO in host mode when dynamic 
 * FIFO sizing is enabled.
 * 16 to 32768 (default 1024)  
 */
/* TODO:Need to check out the exact value. */
#define dwc_param_host_rx_fifo_size_default dwc_param_host_rx_fifo_size
//#if defined(CONFIG_DANUBE)
//#define dwc_param_host_rx_fifo_size_default 512
//#elif defined(CONFIG_AMAZON_SE)
//#define dwc_param_host_rx_fifo_size_default 256
//#else
//#error "Please choose one platform!!"
//#endif

/** Number of 4-byte words in the non-periodic Tx FIFO in host mode 
 * when Dynamic FIFO sizing is enabled in the core. 
 * 16 to 32768 (default 1024)
 */
/* TODO:Need to check out the exact value. */
#define dwc_param_host_nperio_tx_fifo_size_default dwc_param_host_nperio_tx_fifo_size
//#if defined(CONFIG_DANUBE)
//#define dwc_param_host_nperio_tx_fifo_size_default 512
//#elif defined(CONFIG_AMAZON_SE)
//#define dwc_param_host_nperio_tx_fifo_size_default 192
//#else
//#error "Please choose one platform!!"
//#endif

/** Number of 4-byte words in the host periodic Tx FIFO when dynamic 
 * FIFO sizing is enabled. 
 * 16 to 32768 (default 1024)
 */
/* TODO:Need to check out the exact value. */
#define dwc_param_host_perio_tx_fifo_size_default dwc_param_host_perio_tx_fifo_size
//#if defined(CONFIG_DANUBE)
//#define dwc_param_host_perio_tx_fifo_size_default 1024
//#elif defined(CONFIG_AMAZON_SE)
//#define dwc_param_host_perio_tx_fifo_size_default 64
//#else
//#error "Please choose one platform!!"
//#endif

/** The maximum transfer size supported in bytes.  
 * 2047 to 65,535  (default 65,535)
 */
// #define dwc_param_max_transfer_size_default 65535
#define dwc_param_max_transfer_size_default dwc_param_max_transfer_size

/** The maximum number of packets in a transfer.  
 * 15 to 511  (default 511)
 */
//#define dwc_param_max_packet_count_default 511
#define dwc_param_max_packet_count_default dwc_param_max_packet_count

/** The number of host channel registers to use.  
 * 1 to 16 (default 12) 
 * Note: The FPGA configuration supports a maximum of 12 host channels.
 */
// #define dwc_param_host_channels_default 16
#define dwc_param_host_channels_default dwc_param_host_channels

/** The number of endpoints in addition to EP0 available for device 
 * mode operations. 
 * 1 to 15 (default 6 IN and OUT) 
 * Note: The FPGA configuration supports a maximum of 6 IN and OUT 
 * endpoints in addition to EP0.
 */
/* TODO:Need to check out the exact value. */
//#define dwc_param_dev_endpoints_default 6
#define dwc_param_dev_endpoints_default dwc_param_dev_endpoints

/** 
 * Specifies the type of PHY interface to use. By default, the driver
 * will automatically detect the phy_type.
 * 
 * 0 - Full Speed PHY
 * 1 - UTMI+ (default)
 * 2 - ULPI
 */
#define DWC_PHY_TYPE_PARAM_FS 0
#define DWC_PHY_TYPE_PARAM_UTMI 1
#define DWC_PHY_TYPE_PARAM_ULPI 2
//#define dwc_param_phy_type_default DWC_PHY_TYPE_PARAM_UTMI
#define dwc_param_phy_type_default dwc_param_phy_type

/**
 * Specifies the UTMI+ Data Width.  This parameter is
 * applicable for a PHY_TYPE of UTMI+ or ULPI. (For a ULPI
 * PHY_TYPE, this parameter indicates the data width between
 * the MAC and the ULPI Wrapper.) Also, this parameter is
 * applicable only if the OTG_HSPHY_WIDTH cC parameter was set
 * to "8 and 16 bits", meaning that the core has been
 * configured to work at either data path width. 
 *
 * 8 or 16 bits (default 16)
 */
//#define dwc_param_phy_utmi_width_default 16
#define dwc_param_phy_utmi_width_default dwc_param_phy_utmi_width

/**
 * Specifies whether the ULPI operates at double or single
 * data rate. This parameter is only applicable if PHY_TYPE is
 * ULPI.
 * 
 * 0 - single data rate ULPI interface with 8 bit wide data
 * bus (default)
 * 1 - double data rate ULPI interface with 4 bit wide data
 * bus
 */
//#define dwc_param_phy_ulpi_ddr_default 0
#define dwc_param_phy_ulpi_ddr_default dwc_param_phy_ulpi_ddr

/**
 * Specifies whether to use the internal or external supply to 
 * drive the vbus with a ULPI phy.
 */
#define DWC_PHY_ULPI_INTERNAL_VBUS 0
#define DWC_PHY_ULPI_EXTERNAL_VBUS 1
// #define dwc_param_phy_ulpi_ext_vbus_default DWC_PHY_ULPI_INTERNAL_VBUS
#define dwc_param_phy_ulpi_ext_vbus_default dwc_param_phy_ulpi_ext_vbus

/**
 * Specifies whether to use the I2Cinterface for full speed PHY. This
 * parameter is only applicable if PHY_TYPE is FS.
 * 0 - No (default)
 * 1 - Yes
 */
//#define dwc_param_i2c_enable_default 0
#define dwc_param_i2c_enable_default dwc_param_i2c_enable

//#define dwc_param_ulpi_fs_ls_default 0
#define dwc_param_ulpi_fs_ls_default dwc_param_ulpi_fs_ls

//#define dwc_param_ts_dline_default 0
#define dwc_param_ts_dline_default dwc_param_ts_dline


#if defined(CONFIG_DANUBE)
#define mask_and_ack_ifx_irq mask_and_ack_danube_irq
extern void mask_and_ack_danube_irq (unsigned int irq_nr);
#elif defined(CONFIG_AMAZON_SE)
#define mask_and_ack_ifx_irq mask_and_ack_amazon_se_irq
extern void mask_and_ack_amazon_se_irq (unsigned int irq_nr);
#else
#error "Please Choose one platform!!"
#endif

#endif //__DWC_OTG_CIL_IFX_H__

